@@ -9,7 +9,7 @@ use crate::Db;
 use crate::files::{File, FilePath};
 
 /// Reads the source text of a python text file (must be valid UTF8) or notebook.
-#[salsa::tracked(heap_size=get_size2::GetSize::get_heap_size)]
+#[salsa::tracked(heap_size=ruff_memory_usage::heap_size)]
 pub fn source_text(db: &dyn Db, file: File) -> SourceText {
     let path = file.path(db);
     let _span = tracing::trace_span!("source_text", file = %path).entered();
@@ -69,21 +69,21 @@ impl SourceText {
     pub fn as_str(&self) -> &str {
         match &self.inner.kind {
             SourceTextKind::Text(source) => source,
-            SourceTextKind::Notebook(notebook) => notebook.source_code(),
+            SourceTextKind::Notebook { notebook } => notebook.source_code(),
         }
     }
 
     /// Returns the underlying notebook if this is a notebook file.
     pub fn as_notebook(&self) -> Option<&Notebook> {
         match &self.inner.kind {
-            SourceTextKind::Notebook(notebook) => Some(notebook),
+            SourceTextKind::Notebook { notebook } => Some(notebook),
             SourceTextKind::Text(_) => None,
         }
     }
 
     /// Returns `true` if this is a notebook source file.
     pub fn is_notebook(&self) -> bool {
-        matches!(&self.inner.kind, SourceTextKind::Notebook(_))
+        matches!(&self.inner.kind, SourceTextKind::Notebook { .. })
     }
 
     /// Returns `true` if there was an error when reading the content of the file.
@@ -108,7 +108,7 @@ impl std::fmt::Debug for SourceText {
             SourceTextKind::Text(text) => {
                 dbg.field(text);
             }
-            SourceTextKind::Notebook(notebook) => {
+            SourceTextKind::Notebook { notebook } => {
                 dbg.field(notebook);
             }
         }
@@ -123,23 +123,15 @@ struct SourceTextInner {
     read_error: Option<SourceTextError>,
 }
 
-#[derive(Eq, PartialEq)]
+#[derive(Eq, PartialEq, get_size2::GetSize)]
 enum SourceTextKind {
     Text(String),
-    Notebook(Box<Notebook>),
-}
-
-impl get_size2::GetSize for SourceTextKind {
-    fn get_heap_size(&self) -> usize {
-        match self {
-            SourceTextKind::Text(text) => text.get_heap_size(),
-            // TODO: The `get-size` derive does not support ignoring enum variants.
-            //
-            // Jupyter notebooks are not very relevant for memory profiling, and contain
-            // arbitrary JSON values that do not implement the `GetSize` trait.
-            SourceTextKind::Notebook(_) => 0,
-        }
-    }
+    Notebook {
+        // Jupyter notebooks are not very relevant for memory profiling, and contain
+        // arbitrary JSON values that do not implement the `GetSize` trait.
+        #[get_size(ignore)]
+        notebook: Box<Notebook>,
+    },
 }
 
 impl From<String> for SourceTextKind {
@@ -150,7 +142,9 @@ impl From<String> for SourceTextKind {
 
 impl From<Notebook> for SourceTextKind {
     fn from(notebook: Notebook) -> Self {
-        SourceTextKind::Notebook(Box::new(notebook))
+        SourceTextKind::Notebook {
+            notebook: Box::new(notebook),
+        }
     }
 }
 
@@ -163,7 +157,7 @@ pub enum SourceTextError {
 }
 
 /// Computes the [`LineIndex`] for `file`.
-#[salsa::tracked(heap_size=get_size2::GetSize::get_heap_size)]
+#[salsa::tracked(heap_size=ruff_memory_usage::heap_size)]
 pub fn line_index(db: &dyn Db, file: File) -> LineIndex {
     let _span = tracing::trace_span!("line_index", ?file).entered();
 
