@@ -1,6 +1,8 @@
 use crate::checkers::ast::Checker;
 use crate::preview::is_fix_os_path_samefile_enabled;
-use crate::rules::flake8_use_pathlib::helpers::check_os_pathlib_two_arg_calls;
+use crate::rules::flake8_use_pathlib::helpers::{
+    check_os_pathlib_two_arg_calls, has_unknown_keywords_or_starred_expr,
+};
 use crate::{FixAvailability, Violation};
 use ruff_macros::{ViolationMetadata, derive_message_formats};
 use ruff_python_ast::ExprCall;
@@ -65,13 +67,16 @@ pub(crate) fn os_path_samefile(checker: &Checker, call: &ExprCall, segments: &[&
         return;
     }
 
+    let fix_enabled = is_fix_os_path_samefile_enabled(checker.settings())
+        && !has_unknown_keywords_or_starred_expr(&call.arguments, &["f1", "f2"]);
+
     check_os_pathlib_two_arg_calls(
         checker,
         call,
         "samefile",
         "f1",
         "f2",
-        is_fix_os_path_samefile_enabled(checker.settings()),
+        fix_enabled,
         OsPathSamefile,
     );
 }
