@@ -1,10 +1,11 @@
-use ruff_diagnostics::{Diagnostic, Edit, Fix, FixAvailability, Violation};
-use ruff_macros::{derive_message_formats, violation};
+use ruff_macros::{ViolationMetadata, derive_message_formats};
 use ruff_python_trivia::Cursor;
-use ruff_source_file::Locator;
 use ruff_text_size::{Ranged, TextRange};
 
-use crate::noqa::{Directive, NoqaDirectives};
+use crate::Locator;
+use crate::checkers::ast::LintContext;
+use crate::noqa::{self, Directive, FileNoqaDirectives, NoqaDirectives};
+use crate::{Edit, Fix, FixAvailability, Violation};
 
 /// ## What it does
 /// Check for `noqa` annotations that suppress all diagnostics, as opposed to
@@ -37,10 +38,10 @@ use crate::noqa::{Directive, NoqaDirectives};
 ///
 /// ## References
 /// - [Ruff documentation](https://docs.astral.sh/ruff/configuration/#error-suppression)
-#[violation]
-pub struct BlanketNOQA {
+#[derive(ViolationMetadata)]
+pub(crate) struct BlanketNOQA {
     missing_colon: bool,
-    space_before_colon: bool,
+    file_exemption: bool,
 }
 
 impl Violation for BlanketNOQA {
@@ -50,30 +51,22 @@ impl Violation for BlanketNOQA {
     fn message(&self) -> String {
         let BlanketNOQA {
             missing_colon,
-            space_before_colon,
+            file_exemption,
         } = self;
-
         // This awkward branching is necessary to ensure that the generic message is picked up by
         // `derive_message_formats`.
-        if !missing_colon && !space_before_colon {
-            format!("Use specific rule codes when using `noqa`")
-        } else if *missing_colon {
-            format!("Use a colon when specifying `noqa` rule codes")
+        if !missing_colon && !file_exemption {
+            "Use specific rule codes when using `noqa`".to_string()
+        } else if *file_exemption {
+            "Use specific rule codes when using `ruff: noqa`".to_string()
         } else {
-            format!("Do not add spaces between `noqa` and its colon")
+            "Use a colon when specifying `noqa` rule codes".to_string()
         }
     }
 
     fn fix_title(&self) -> Option<String> {
-        let BlanketNOQA {
-            missing_colon,
-            space_before_colon,
-        } = self;
-
-        if *missing_colon {
+        if self.missing_colon {
             Some("Add missing colon".to_string())
-        } else if *space_before_colon {
-            Some("Remove space(s) before colon".to_string())
         } else {
             None
         }
@@ -82,10 +75,23 @@ impl Violation for BlanketNOQA {
 
 /// PGH004
 pub(crate) fn blanket_noqa(
-    diagnostics: &mut Vec<Diagnostic>,
+    context: &LintContext,
     noqa_directives: &NoqaDirectives,
     locator: &Locator,
+    file_noqa_directives: &FileNoqaDirectives,
 ) {
+    for line in file_noqa_directives.lines() {
+        if let Directive::All(_) = line.parsed_file_exemption {
+            context.report_diagnostic(
+                BlanketNOQA {
+                    missing_colon: false,
+                    file_exemption: true,
+                },
+                line.range(),
+            );
+        }
+    }
+
     for directive_line in noqa_directives.lines() {
         if let Directive::All(all) = &directive_line.directive {
             let line = locator.slice(directive_line);
@@ -95,43 +101,28 @@ pub(crate) fn blanket_noqa(
             let mut cursor = Cursor::new(&line[noqa_end.to_usize()..]);
             cursor.eat_while(char::is_whitespace);
 
-            // Check for extraneous spaces before the colon.
-            // Ex) `# noqa : F401`
-            if cursor.first() == ':' {
-                let start = all.end();
-                let end = start + cursor.token_len();
-                let mut diagnostic = Diagnostic::new(
-                    BlanketNOQA {
-                        missing_colon: false,
-                        space_before_colon: true,
-                    },
-                    TextRange::new(all.start(), end),
-                );
-                diagnostic.set_fix(Fix::unsafe_edit(Edit::deletion(start, end)));
-                diagnostics.push(diagnostic);
-            } else if Directive::lex_code(cursor.chars().as_str()).is_some() {
+            if noqa::lex_codes(cursor.chars().as_str()).is_ok_and(|codes| !codes.is_empty()) {
                 // Check for a missing colon.
                 // Ex) `# noqa F401`
                 let start = all.end();
                 let end = start + cursor.token_len();
-                let mut diagnostic = Diagnostic::new(
+                let mut diagnostic = context.report_diagnostic(
                     BlanketNOQA {
                         missing_colon: true,
-                        space_before_colon: false,
+                        file_exemption: false,
                     },
                     TextRange::new(all.start(), end),
                 );
                 diagnostic.set_fix(Fix::unsafe_edit(Edit::insertion(':'.to_string(), start)));
-                diagnostics.push(diagnostic);
             } else {
                 // Otherwise, it looks like an intentional blanket `noqa` annotation.
-                diagnostics.push(Diagnostic::new(
+                context.report_diagnostic(
                     BlanketNOQA {
                         missing_colon: false,
-                        space_before_colon: false,
+                        file_exemption: false,
                     },
                     all.range(),
-                ));
+                );
             }
         }
     }
