@@ -1,8 +1,8 @@
-use ruff_diagnostics::{Diagnostic, Violation};
-use ruff_macros::{derive_message_formats, violation};
+use ruff_macros::{ViolationMetadata, derive_message_formats};
 use ruff_python_ast::Stmt;
 use ruff_text_size::Ranged;
 
+use crate::Violation;
 use crate::checkers::ast::Checker;
 
 /// ## What it does
@@ -14,21 +14,19 @@ use crate::checkers::ast::Checker;
 ///
 /// At the module level, all names are global by default, so the `global`
 /// keyword is redundant.
-#[violation]
-pub struct GlobalAtModuleLevel;
+#[derive(ViolationMetadata)]
+pub(crate) struct GlobalAtModuleLevel;
 
 impl Violation for GlobalAtModuleLevel {
     #[derive_message_formats]
     fn message(&self) -> String {
-        format!("`global` at module level is redundant")
+        "`global` at module level is redundant".to_string()
     }
 }
 
 /// PLW0604
-pub(crate) fn global_at_module_level(checker: &mut Checker, stmt: &Stmt) {
+pub(crate) fn global_at_module_level(checker: &Checker, stmt: &Stmt) {
     if checker.semantic().current_scope().kind.is_module() {
-        checker
-            .diagnostics
-            .push(Diagnostic::new(GlobalAtModuleLevel, stmt.range()));
+        checker.report_diagnostic(GlobalAtModuleLevel, stmt.range());
     }
 }
