@@ -1,12 +1,12 @@
-use ruff_diagnostics::{Diagnostic, Edit, Fix, FixAvailability, Violation};
-use ruff_macros::{derive_message_formats, violation};
+use ruff_macros::{ViolationMetadata, derive_message_formats};
 use ruff_python_ast as ast;
-use ruff_python_semantic::analyze::function_type;
 use ruff_python_semantic::Scope;
+use ruff_python_semantic::analyze::function_type;
 use ruff_text_size::Ranged;
 
 use crate::checkers::ast::Checker;
 use crate::importer::ImportRequest;
+use crate::{Edit, Fix, FixAvailability, Violation};
 
 /// ## What it does
 /// Checks for methods decorated with `@singledispatch`.
@@ -18,39 +18,39 @@ use crate::importer::ImportRequest;
 /// standalone function.
 ///
 /// ## Example
+///
 /// ```python
 /// from functools import singledispatch
 ///
 ///
 /// class Class:
 ///     @singledispatch
-///     def method(self, arg):
-///         ...
+///     def method(self, arg): ...
 /// ```
 ///
 /// Use instead:
+///
 /// ```python
 /// from functools import singledispatchmethod
 ///
 ///
 /// class Class:
 ///     @singledispatchmethod
-///     def method(self, arg):
-///         ...
+///     def method(self, arg): ...
 /// ```
 ///
 /// ## Fix safety
 /// This rule's fix is marked as unsafe, as migrating from `@singledispatch` to
 /// `@singledispatchmethod` may change the behavior of the code.
-#[violation]
-pub struct SingledispatchMethod;
+#[derive(ViolationMetadata)]
+pub(crate) struct SingledispatchMethod;
 
 impl Violation for SingledispatchMethod {
     const FIX_AVAILABILITY: FixAvailability = FixAvailability::Sometimes;
 
     #[derive_message_formats]
     fn message(&self) -> String {
-        format!("`@singledispatch` decorator should not be used on methods")
+        "`@singledispatch` decorator should not be used on methods".to_string()
     }
 
     fn fix_title(&self) -> Option<String> {
@@ -58,12 +58,8 @@ impl Violation for SingledispatchMethod {
     }
 }
 
-/// E1519
-pub(crate) fn singledispatch_method(
-    checker: &Checker,
-    scope: &Scope,
-    diagnostics: &mut Vec<Diagnostic>,
-) {
+/// PLE1519
+pub(crate) fn singledispatch_method(checker: &Checker, scope: &Scope) {
     let Some(func) = scope.kind.as_function() else {
         return;
     };
@@ -74,7 +70,7 @@ pub(crate) fn singledispatch_method(
         ..
     } = func;
 
-    let Some(parent) = &checker.semantic().first_non_type_parent_scope(scope) else {
+    let Some(parent) = checker.semantic().first_non_type_parent_scope(scope) else {
         return;
     };
 
@@ -83,8 +79,8 @@ pub(crate) fn singledispatch_method(
         decorator_list,
         parent,
         checker.semantic(),
-        &checker.settings.pep8_naming.classmethod_decorators,
-        &checker.settings.pep8_naming.staticmethod_decorators,
+        &checker.settings().pep8_naming.classmethod_decorators,
+        &checker.settings().pep8_naming.staticmethod_decorators,
     );
     if !matches!(
         type_,
@@ -103,7 +99,7 @@ pub(crate) fn singledispatch_method(
                 matches!(qualified_name.segments(), ["functools", "singledispatch"])
             })
         {
-            let mut diagnostic = Diagnostic::new(SingledispatchMethod, decorator.range());
+            let mut diagnostic = checker.report_diagnostic(SingledispatchMethod, decorator.range());
             diagnostic.try_set_fix(|| {
                 let (import_edit, binding) = checker.importer().get_or_import_symbol(
                     &ImportRequest::import("functools", "singledispatchmethod"),
@@ -115,7 +111,6 @@ pub(crate) fn singledispatch_method(
                     [import_edit],
                 ))
             });
-            diagnostics.push(diagnostic);
         }
     }
 }
