@@ -1,15 +1,19 @@
+use ruff_diagnostics::{Applicability, Edit, Fix};
 use ruff_macros::{ViolationMetadata, derive_message_formats};
-use ruff_python_ast::relocate::relocate_expr;
-use ruff_python_ast::visitor::{self, Visitor};
-use ruff_python_ast::{self as ast, Expr, Stmt};
+use ruff_python_ast::{
+    self as ast, Expr, Stmt,
+    relocate::relocate_expr,
+    visitor::{self, Visitor},
+};
+
 use ruff_python_codegen::Generator;
 use ruff_text_size::{Ranged, TextRange};
 
-use crate::Violation;
 use crate::checkers::ast::Checker;
 use crate::fix::snippet::SourceCodeSnippet;
-
+use crate::importer::ImportRequest;
 use crate::rules::refurb::helpers::{FileOpen, find_file_opens};
+use crate::{FixAvailability, Violation};
 
 /// ## What it does
 /// Checks for uses of `open` and `write` that can be replaced by `pathlib`
@@ -33,6 +37,9 @@ use crate::rules::refurb::helpers::{FileOpen, find_file_opens};
 /// Path(filename).write_text(contents)
 /// ```
 ///
+/// ## Fix Safety
+/// This rule's fix is marked as unsafe if the replacement would remove comments attached to the original expression.
+///
 /// ## References
 /// - [Python documentation: `Path.write_bytes`](https://docs.python.org/3/library/pathlib.html#pathlib.Path.write_bytes)
 /// - [Python documentation: `Path.write_text`](https://docs.python.org/3/library/pathlib.html#pathlib.Path.write_text)
@@ -43,12 +50,21 @@ pub(crate) struct WriteWholeFile {
 }
 
 impl Violation for WriteWholeFile {
+    const FIX_AVAILABILITY: FixAvailability = FixAvailability::Sometimes;
+
     #[derive_message_formats]
     fn message(&self) -> String {
         let filename = self.filename.truncated_display();
         let suggestion = self.suggestion.truncated_display();
         format!("`open` and `write` should be replaced by `Path({filename}).{suggestion}`")
     }
+    fn fix_title(&self) -> Option<String> {
+        Some(format!(
+            "Replace with `Path({}).{}`",
+            self.filename.truncated_display(),
+            self.suggestion.truncated_display(),
+        ))
+    }
 }
 
 /// FURB103
@@ -65,7 +81,7 @@ pub(crate) fn write_whole_file(checker: &Checker, with: &ast::StmtWith) {
     }
 
     // Then we need to match each `open` operation with exactly one `write` call.
-    let mut matcher = WriteMatcher::new(checker, candidates);
+    let mut matcher = WriteMatcher::new(checker, candidates, with);
     visitor::walk_body(&mut matcher, &with.body);
 }
 
@@ -74,21 +90,27 @@ struct WriteMatcher<'a, 'b> {
     checker: &'a Checker<'b>,
     candidates: Vec<FileOpen<'a>>,
     loop_counter: u32,
+    with_stmt: &'a ast::StmtWith,
 }
 
 impl<'a, 'b> WriteMatcher<'a, 'b> {
-    fn new(checker: &'a Checker<'b>, candidates: Vec<FileOpen<'a>>) -> Self {
+    fn new(
+        checker: &'a Checker<'b>,
+        candidates: Vec<FileOpen<'a>>,
+        with_stmt: &'a ast::StmtWith,
+    ) -> Self {
         Self {
             checker,
             candidates,
             loop_counter: 0,
+            with_stmt,
         }
     }
 }
 
 impl<'a> Visitor<'a> for WriteMatcher<'a, '_> {
     fn visit_stmt(&mut self, stmt: &'a Stmt) {
-        if matches!(stmt, ast::Stmt::While(_) | ast::Stmt::For(_)) {
+        if matches!(stmt, Stmt::While(_) | Stmt::For(_)) {
             self.loop_counter += 1;
             visitor::walk_stmt(self, stmt);
             self.loop_counter -= 1;
@@ -104,19 +126,30 @@ impl<'a> Visitor<'a> for WriteMatcher<'a, '_> {
                 .iter()
                 .position(|open| open.is_ref(write_to))
             {
+                let open = self.candidates.remove(open);
+
                 if self.loop_counter == 0 {
-                    let open = self.candidates.remove(open);
-                    self.checker.report_diagnostic(
+                    let suggestion = make_suggestion(&open, content, self.checker.generator());
+
+                    let mut diagnostic = self.checker.report_diagnostic(
                         WriteWholeFile {
                             filename: SourceCodeSnippet::from_str(
                                 &self.checker.generator().expr(open.filename),
                             ),
-                            suggestion: make_suggestion(&open, content, self.checker.generator()),
+                            suggestion: SourceCodeSnippet::from_str(&suggestion),
                         },
                         open.item.range(),
                     );
-                } else {
-                    self.candidates.remove(open);
+
+                    if !crate::preview::is_fix_write_whole_file_enabled(self.checker.settings()) {
+                        return;
+                    }
+
+                    if let Some(fix) =
+                        generate_fix(self.checker, &open, self.with_stmt, &suggestion)
+                    {
+                        diagnostic.set_fix(fix);
+                    }
                 }
             }
             return;
@@ -143,7 +176,7 @@ fn match_write_call(expr: &Expr) -> Option<(&Expr, &Expr)> {
     Some((&*attr.value, call.arguments.args.first()?))
 }
 
-fn make_suggestion(open: &FileOpen<'_>, arg: &Expr, generator: Generator) -> SourceCodeSnippet {
+fn make_suggestion(open: &FileOpen<'_>, arg: &Expr, generator: Generator) -> String {
     let name = ast::ExprName {
         id: open.mode.pathlib_method(),
         ctx: ast::ExprContext::Load,
@@ -163,5 +196,42 @@ fn make_suggestion(open: &FileOpen<'_>, arg: &Expr, generator: Generator) -> Sou
         range: TextRange::default(),
         node_index: ruff_python_ast::AtomicNodeIndex::NONE,
     };
-    SourceCodeSnippet::from_str(&generator.expr(&call.into()))
+    generator.expr(&call.into())
+}
+
+fn generate_fix(
+    checker: &Checker,
+    open: &FileOpen,
+    with_stmt: &ast::StmtWith,
+    suggestion: &str,
+) -> Option<Fix> {
+    if !(with_stmt.items.len() == 1 && matches!(with_stmt.body.as_slice(), [Stmt::Expr(_)])) {
+        return None;
+    }
+
+    let locator = checker.locator();
+    let filename_code = locator.slice(open.filename.range());
+
+    let (import_edit, binding) = checker
+        .importer()
+        .get_or_import_symbol(
+            &ImportRequest::import("pathlib", "Path"),
+            with_stmt.start(),
+            checker.semantic(),
+        )
+        .ok()?;
+
+    let replacement = format!("{binding}({filename_code}).{suggestion}");
+
+    let applicability = if checker.comment_ranges().intersects(with_stmt.range()) {
+        Applicability::Unsafe
+    } else {
+        Applicability::Safe
+    };
+
+    Some(Fix::applicable_edits(
+        Edit::range_replacement(replacement, with_stmt.range()),
+        [import_edit],
+        applicability,
+    ))
 }
