@@ -3,25 +3,26 @@
 #![allow(clippy::doc_markdown)]
 
 use std::cmp::Ordering;
+use std::sync::LazyLock;
 use std::{borrow::Cow, collections::VecDeque};
 
 use itertools::Itertools;
+use regex::Regex;
 
 use ruff_formatter::printer::SourceMapGeneration;
-use ruff_python_ast::{str::Quote, StringFlags};
+use ruff_python_ast::{AnyStringFlags, StringFlags, str::Quote};
+use ruff_python_parser::ParseOptions;
 use ruff_python_trivia::CommentRanges;
-use {once_cell::sync::Lazy, regex::Regex};
 use {
-    ruff_formatter::{write, FormatOptions, IndentStyle, LineWidth, Printed},
-    ruff_python_trivia::{is_python_whitespace, PythonWhitespace},
-    ruff_source_file::Locator,
+    ruff_formatter::{FormatOptions, IndentStyle, LineWidth, Printed, write},
+    ruff_python_trivia::{PythonWhitespace, is_python_whitespace},
     ruff_text_size::{Ranged, TextLen, TextRange, TextSize},
 };
 
-use crate::string::StringQuotes;
-use crate::{prelude::*, DocstringCodeLineWidth, FormatModuleError};
-
 use super::NormalizedString;
+use crate::preview::is_no_chaperone_for_escaped_quote_in_triple_quoted_docstring_enabled;
+use crate::string::StringQuotes;
+use crate::{DocstringCodeLineWidth, FormatModuleError, prelude::*};
 
 /// Format a docstring by trimming whitespace and adjusting the indentation.
 ///
@@ -167,7 +168,7 @@ pub(crate) fn format(normalized: &NormalizedString, f: &mut PyFormatter) -> Form
     if docstring[first.len()..].trim().is_empty() {
         // For `"""\n"""` or other whitespace between the quotes, black keeps a single whitespace,
         // but `""""""` doesn't get one inserted.
-        if needs_chaperone_space(normalized, trim_end)
+        if needs_chaperone_space(normalized.flags(), trim_end, f.context())
             || (trim_end.is_empty() && !docstring.is_empty())
         {
             space().fmt(f)?;
@@ -189,7 +190,7 @@ pub(crate) fn format(normalized: &NormalizedString, f: &mut PyFormatter) -> Form
         // We don't want to count whitespace-only lines as miss-indented
         .filter(|line| !line.trim().is_empty())
         .map(Indentation::from_str)
-        .min_by_key(|indentation| indentation.width())
+        .min_by_key(|indentation| indentation.columns())
         .unwrap_or_default();
 
     DocstringLinePrinter {
@@ -207,7 +208,7 @@ pub(crate) fn format(normalized: &NormalizedString, f: &mut PyFormatter) -> Form
     let trim_end = docstring
         .as_ref()
         .trim_end_matches(|c: char| c.is_whitespace() && c != '\n');
-    if needs_chaperone_space(normalized, trim_end) {
+    if needs_chaperone_space(normalized.flags(), trim_end, f.context()) {
         space().fmt(f)?;
     }
 
@@ -263,7 +264,7 @@ struct DocstringLinePrinter<'ast, 'buf, 'fmt, 'src> {
     code_example: CodeExample<'src>,
 }
 
-impl<'ast, 'buf, 'fmt, 'src> DocstringLinePrinter<'ast, 'buf, 'fmt, 'src> {
+impl<'src> DocstringLinePrinter<'_, '_, '_, 'src> {
     /// Print all of the lines in the given iterator to this
     /// printer's formatter.
     ///
@@ -353,7 +354,7 @@ impl<'ast, 'buf, 'fmt, 'src> DocstringLinePrinter<'ast, 'buf, 'fmt, 'src> {
                             };
                             // This looks suspicious, but it's consistent with the whitespace
                             // normalization that will occur anyway.
-                            let indent = " ".repeat(min_indent.width());
+                            let indent = " ".repeat(min_indent.columns());
                             for docline in formatted_lines {
                                 self.print_one(
                                     &docline.map(|line| std::format!("{indent}{line}")),
@@ -363,7 +364,7 @@ impl<'ast, 'buf, 'fmt, 'src> DocstringLinePrinter<'ast, 'buf, 'fmt, 'src> {
                         CodeExampleKind::Markdown(fenced) => {
                             // This looks suspicious, but it's consistent with the whitespace
                             // normalization that will occur anyway.
-                            let indent = " ".repeat(fenced.opening_fence_indent.width());
+                            let indent = " ".repeat(fenced.opening_fence_indent.columns());
                             for docline in formatted_lines {
                                 self.print_one(
                                     &docline.map(|line| std::format!("{indent}{line}")),
@@ -455,10 +456,10 @@ impl<'ast, 'buf, 'fmt, 'src> DocstringLinePrinter<'ast, 'buf, 'fmt, 'src> {
             // (see example in [`format_docstring`] doc comment). We then
             // prepend the in-docstring indentation to the string.
             let indent_len =
-                Indentation::from_str(trim_end).width() - self.stripped_indentation.width();
+                Indentation::from_str(trim_end).columns() - self.stripped_indentation.columns();
             let in_docstring_indent = " ".repeat(indent_len) + trim_end.trim_start();
             text(&in_docstring_indent).fmt(self.f)?;
-        };
+        }
 
         // We handled the case that the closing quotes are on their own line
         // above (the last line is empty except for whitespace). If they are on
@@ -492,19 +493,28 @@ impl<'ast, 'buf, 'fmt, 'src> DocstringLinePrinter<'ast, 'buf, 'fmt, 'src> {
         &mut self,
         kind: &mut CodeExampleKind<'_>,
     ) -> FormatResult<Option<Vec<OutputDocstringLine<'static>>>> {
-        use ruff_python_parser::AsMode;
-
         let line_width = match self.f.options().docstring_code_line_width() {
             DocstringCodeLineWidth::Fixed(width) => width,
             DocstringCodeLineWidth::Dynamic => {
                 let global_line_width = self.f.options().line_width().value();
                 let indent_width = self.f.options().indent_width();
                 let indent_level = self.f.context().indent_level();
-                let current_indent = indent_level
+                let mut current_indent = indent_level
                     .to_ascii_spaces(indent_width)
                     .saturating_add(kind.extra_indent_ascii_spaces());
+
+                // Add the in-docstring indentation
+                current_indent = current_indent.saturating_add(
+                    u16::try_from(
+                        kind.indent()
+                            .columns()
+                            .saturating_sub(self.stripped_indentation.columns()),
+                    )
+                    .unwrap_or(u16::MAX),
+                );
+
                 let width = std::cmp::max(1, global_line_width.saturating_sub(current_indent));
-                LineWidth::try_from(width).expect("width is capped at a minimum of 1")
+                LineWidth::try_from(width).expect("width should be capped at a minimum of 1")
             }
         };
 
@@ -559,7 +569,8 @@ impl<'ast, 'buf, 'fmt, 'src> DocstringLinePrinter<'ast, 'buf, 'fmt, 'src> {
                 std::format!(r#""""{}""""#, printed.as_code())
             }
         };
-        let result = ruff_python_parser::parse(&wrapped, self.f.options().source_type().as_mode());
+        let result =
+            ruff_python_parser::parse(&wrapped, ParseOptions::from(self.f.options().source_type()));
         // If the resulting code is not valid, then reset and pass through
         // the docstring lines as-is.
         if result.is_err() {
@@ -648,7 +659,7 @@ struct OutputDocstringLine<'src> {
     is_last: bool,
 }
 
-impl<'src> OutputDocstringLine<'src> {
+impl OutputDocstringLine<'_> {
     /// Return this reformatted line, but with the given function applied to
     /// the text of the line.
     fn map(self, mut map: impl FnMut(&str) -> String) -> OutputDocstringLine<'static> {
@@ -828,6 +839,26 @@ impl<'src> CodeExampleKind<'src> {
             _ => 0,
         }
     }
+
+    /// The indent of the entire code block relative to the start of the line.
+    ///
+    /// For example:
+    /// ```python
+    /// def test():
+    ///     """Docstring
+    ///     Example:
+    ///        >>> 1 + 1
+    /// ```
+    ///
+    /// The `>>> ` block has an indent of 8 columns: The shared indent with the docstring and the 4 spaces
+    /// inside the docstring.
+    fn indent(&self) -> Indentation {
+        match self {
+            CodeExampleKind::Doctest(doctest) => Indentation::from_str(doctest.ps1_indent),
+            CodeExampleKind::Rst(rst) => rst.min_indent.unwrap_or(rst.opening_indent),
+            CodeExampleKind::Markdown(markdown) => markdown.opening_fence_indent,
+        }
+    }
 }
 
 /// State corresponding to a single doctest code example found in a docstring.
@@ -928,9 +959,9 @@ impl<'src> CodeExampleDoctest<'src> {
 /// the same with two main differences:
 ///
 /// 1. Literal blocks are began with a line that ends with `::`. Code block
-/// directives are began with a line like `.. code-block:: python`.
+///    directives are began with a line like `.. code-block:: python`.
 /// 2. Code block directives permit a list of options as a "field list"
-/// immediately after the opening line. Literal blocks have no options.
+///    immediately after the opening line. Literal blocks have no options.
 ///
 /// Otherwise, everything else, including the indentation structure, is the
 /// same.
@@ -989,7 +1020,7 @@ impl<'src> CodeExampleRst<'src> {
     ///
     /// [literal block]: https://docutils.sourceforge.io/docs/ref/rst/restructuredtext.html#literal-blocks
     /// [code block directive]: https://www.sphinx-doc.org/en/master/usage/restructuredtext/directives.html#directive-code-block
-    fn new(original: InputDocstringLine<'src>) -> Option<CodeExampleRst> {
+    fn new(original: InputDocstringLine<'src>) -> Option<CodeExampleRst<'src>> {
         let (opening_indent, rest) = indent_with_suffix(original.line);
         if rest.starts_with(".. ") {
             if let Some(litblock) = CodeExampleRst::new_code_block(original) {
@@ -1024,7 +1055,7 @@ impl<'src> CodeExampleRst<'src> {
     /// Attempts to create a new reStructuredText code example from a
     /// `code-block` or `sourcecode` directive. If one couldn't be found, then
     /// `None` is returned.
-    fn new_code_block(original: InputDocstringLine<'src>) -> Option<CodeExampleRst> {
+    fn new_code_block(original: InputDocstringLine<'src>) -> Option<CodeExampleRst<'src>> {
         // This regex attempts to parse the start of a reStructuredText code
         // block [directive]. From the reStructuredText spec:
         //
@@ -1042,7 +1073,7 @@ impl<'src> CodeExampleRst<'src> {
         // [directives]: https://docutils.sourceforge.io/docs/ref/rst/restructuredtext.html#directives
         // [Pygments lexer names]: https://pygments.org/docs/lexers/
         // [code-block]: https://www.sphinx-doc.org/en/master/usage/restructuredtext/directives.html#directive-code-block
-        static DIRECTIVE_START: Lazy<Regex> = Lazy::new(|| {
+        static DIRECTIVE_START: LazyLock<Regex> = LazyLock::new(|| {
             Regex::new(
                 r"(?m)^\s*\.\. \s*(?i:code-block|sourcecode)::\s*(?i:python|py|python3|py3)$",
             )
@@ -1287,7 +1318,7 @@ impl<'src> CodeExampleMarkdown<'src> {
     ///
     /// [fenced code block]: https://spec.commonmark.org/0.30/#fenced-code-blocks
     fn new(original: InputDocstringLine<'src>) -> Option<CodeExampleMarkdown<'src>> {
-        static FENCE_START: Lazy<Regex> = Lazy::new(|| {
+        static FENCE_START: LazyLock<Regex> = LazyLock::new(|| {
             Regex::new(
                 r"(?xm)
                 ^
@@ -1549,16 +1580,13 @@ fn docstring_format_source(
     docstring_quote_style: Quote,
     source: &str,
 ) -> Result<Printed, FormatModuleError> {
-    use ruff_python_parser::AsMode;
-
     let source_type = options.source_type();
-    let parsed = ruff_python_parser::parse(source, source_type.as_mode())?;
+    let parsed = ruff_python_parser::parse(source, ParseOptions::from(source_type))?;
     let comment_ranges = CommentRanges::from(parsed.tokens());
     let source_code = ruff_formatter::SourceCode::new(source);
     let comments = crate::Comments::from_ast(parsed.syntax(), source_code, &comment_ranges);
-    let locator = Locator::new(source);
 
-    let ctx = PyFormatContext::new(options, locator.contents(), comments, parsed.tokens())
+    let ctx = PyFormatContext::new(options, source, comments, parsed.tokens())
         .in_docstring(docstring_quote_style);
     let formatted = crate::format!(ctx, [parsed.syntax().format()])?;
     formatted
@@ -1568,12 +1596,43 @@ fn docstring_format_source(
     Ok(formatted.print()?)
 }
 
-/// If the last line of the docstring is `content" """` or `content\ """`, we need a chaperone space
-/// that avoids `content""""` and `content\"""`. This does only applies to un-escaped backslashes,
-/// so `content\\ """` doesn't need a space while `content\\\ """` does.
-fn needs_chaperone_space(normalized: &NormalizedString, trim_end: &str) -> bool {
-    trim_end.ends_with(normalized.flags().quote_style().as_char())
-        || trim_end.chars().rev().take_while(|c| *c == '\\').count() % 2 == 1
+/// If the last line of the docstring is `content""""` or `content\"""`, we need a chaperone space
+/// that avoids `content""""` and `content\"""`. This only applies to un-escaped backslashes,
+/// so `content\\"""` doesn't need a space while `content\\\"""` does.
+pub(super) fn needs_chaperone_space(
+    flags: AnyStringFlags,
+    trim_end: &str,
+    context: &PyFormatContext,
+) -> bool {
+    if count_consecutive_chars_from_end(trim_end, '\\') % 2 == 1 {
+        // Odd backslash count; chaperone avoids escaping closing quotes
+        // `"\ "` -> prevent that this becomes `"\"` which escapes the closing quote.
+        return true;
+    }
+
+    if is_no_chaperone_for_escaped_quote_in_triple_quoted_docstring_enabled(context) {
+        if flags.is_triple_quoted() {
+            if let Some(before_quote) = trim_end.strip_suffix(flags.quote_style().as_char()) {
+                if count_consecutive_chars_from_end(before_quote, '\\').is_multiple_of(2) {
+                    // Even backslash count preceding quote;
+                    // ```py
+                    // """a "  """
+                    // """a \\"  """
+                    // ```
+                    // The chaperon is needed or the triple quoted string "ends" with 4 instead of 3 quotes.
+                    return true;
+                }
+            }
+        }
+
+        false
+    } else {
+        flags.is_triple_quoted() && trim_end.ends_with(flags.quote_style().as_char())
+    }
+}
+
+fn count_consecutive_chars_from_end(s: &str, target: char) -> usize {
+    s.chars().rev().take_while(|c| *c == target).count()
 }
 
 #[derive(Copy, Clone, Debug)]
@@ -1663,7 +1722,7 @@ impl Indentation {
     /// to the next multiple of 8. This is effectively a port of
     /// [`str.expandtabs`](https://docs.python.org/3/library/stdtypes.html#str.expandtabs),
     /// which black [calls with the default tab width of 8](https://github.com/psf/black/blob/c36e468794f9256d5e922c399240d49782ba04f1/src/black/strings.py#L61).
-    const fn width(self) -> usize {
+    const fn columns(self) -> usize {
         match self {
             Self::Spaces(count) => count,
             Self::Tabs(count) => count * Self::TAB_INDENT_WIDTH,
@@ -1737,7 +1796,7 @@ impl Indentation {
                     }),
 
                     _ => None,
-                }
+                };
             }
             Self::Mixed { .. } => return None,
         };
@@ -1769,7 +1828,7 @@ impl Indentation {
     fn trim_start_str(self, line: &str) -> &str {
         let mut seen_indent_len = 0;
         let mut trimmed = line;
-        let indent_len = self.width();
+        let indent_len = self.columns();
 
         for char in line.chars() {
             if seen_indent_len >= indent_len {
@@ -1797,13 +1856,13 @@ impl Indentation {
 
 impl PartialOrd for Indentation {
     fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
-        Some(self.width().cmp(&other.width()))
+        Some(self.columns().cmp(&other.columns()))
     }
 }
 
 impl PartialEq for Indentation {
     fn eq(&self, other: &Self) -> bool {
-        self.width() == other.width()
+        self.columns() == other.columns()
     }
 }
 
@@ -1843,10 +1902,10 @@ mod tests {
     use crate::string::docstring::Indentation;
 
     #[test]
-    fn test_indentation_like_black() {
-        assert_eq!(Indentation::from_str("\t \t  \t").width(), 24);
-        assert_eq!(Indentation::from_str("\t        \t").width(), 24);
-        assert_eq!(Indentation::from_str("\t\t\t").width(), 24);
-        assert_eq!(Indentation::from_str("    ").width(), 4);
+    fn indentation_like_black() {
+        assert_eq!(Indentation::from_str("\t \t  \t").columns(), 24);
+        assert_eq!(Indentation::from_str("\t        \t").columns(), 24);
+        assert_eq!(Indentation::from_str("\t\t\t").columns(), 24);
+        assert_eq!(Indentation::from_str("    ").columns(), 4);
     }
 }
