@@ -943,9 +943,10 @@ fn defaults_to_a_new_python_version() -> anyhow::Result<()> {
 /// The `site-packages` directory is used by ty for external import.
 /// Ty does the following checks to discover the `site-packages` directory in the order:
 /// 1) If `VIRTUAL_ENV` environment variable is set
-/// 2) If `CONDA_PREFIX` environment variable is set (and .filename != `CONDA_DEFAULT_ENV`)
+/// 2) If `CONDA_PREFIX` environment variable is set (and .filename == `CONDA_DEFAULT_ENV`)
 /// 3) If a `.venv` directory exists at the project root
-/// 4) If `CONDA_PREFIX` environment variable is set (and .filename == `CONDA_DEFAULT_ENV`)
+/// 4) If `CONDA_PREFIX` environment variable is set (and .filename != `CONDA_DEFAULT_ENV`)
+///    or if `_CONDA_ROOT` is set (and `_CONDA_ROOT` == `CONDA_PREFIX`)
 ///
 /// This test (and the next one) is aiming at validating the logic around these cases.
 ///
@@ -986,15 +987,14 @@ fn defaults_to_a_new_python_version() -> anyhow::Result<()> {
 /// │           └── site-packages
 /// │               └── package1
 /// │                   └── __init__.py
-/// ├── conda-env
-/// │   └── lib
-/// │       └── python3.13
-/// │           └── site-packages
-/// │               └── package1
-/// │                   └── __init__.py
 /// └── conda
+///     ├── lib
+///     │   └── python3.13
+///     │       └── site-packages
+///     │           └── package1
+///     │               └── __init__.py
 ///     └── envs
-///         └── base
+///         └── conda-env
 ///             └── lib
 ///                 └── python3.13
 ///                     └── site-packages
@@ -1006,15 +1006,15 @@ fn defaults_to_a_new_python_version() -> anyhow::Result<()> {
 #[test]
 fn check_venv_resolution_with_working_venv() -> anyhow::Result<()> {
     let child_conda_package1_path = if cfg!(windows) {
-        "conda-env/Lib/site-packages/package1/__init__.py"
+        "conda/envs/conda-env/Lib/site-packages/package1/__init__.py"
     } else {
-        "conda-env/lib/python3.13/site-packages/package1/__init__.py"
+        "conda/envs/conda-env/lib/python3.13/site-packages/package1/__init__.py"
     };
 
     let base_conda_package1_path = if cfg!(windows) {
-        "conda/envs/base/Lib/site-packages/package1/__init__.py"
+        "conda/Lib/site-packages/package1/__init__.py"
     } else {
-        "conda/envs/base/lib/python3.13/site-packages/package1/__init__.py"
+        "conda//lib/python3.13/site-packages/package1/__init__.py"
     };
 
     let working_venv_package1_path = if cfg!(windows) {
@@ -1136,7 +1136,7 @@ home = ./
     // run with CONDA_PREFIX set, should find the child conda
     assert_cmd_snapshot!(case.command()
         .current_dir(case.root().join("project"))
-        .env("CONDA_PREFIX", case.root().join("conda-env")), @r"
+        .env("CONDA_PREFIX", case.root().join("conda/envs/conda-env")), @r"
     success: false
     exit_code: 1
     ----- stdout -----
@@ -1157,61 +1157,10 @@ home = ./
     WARN ty is pre-release software and not ready for production use. Expect to encounter bugs, missing features, and fatal errors.
     ");
 
-    // run with CONDA_PREFIX and CONDA_DEFAULT_ENV set (unequal), should find child conda
+    // run with CONDA_PREFIX and CONDA_DEFAULT_ENV set (unequal), should find working venv
     assert_cmd_snapshot!(case.command()
         .current_dir(case.root().join("project"))
-        .env("CONDA_PREFIX", case.root().join("conda-env"))
-        .env("CONDA_DEFAULT_ENV", "base"), @r"
-    success: false
-    exit_code: 1
-    ----- stdout -----
-    error[unresolved-import]: Module `package1` has no member `ChildConda`
-     --> test.py:3:22
-      |
-    2 | from package1 import ActiveVenv
-    3 | from package1 import ChildConda
-      |                      ^^^^^^^^^^
-    4 | from package1 import WorkingVenv
-    5 | from package1 import BaseConda
-      |
-    info: rule `unresolved-import` is enabled by default
-
-    Found 1 diagnostic
-
-    ----- stderr -----
-    WARN ty is pre-release software and not ready for production use. Expect to encounter bugs, missing features, and fatal errors.
-    ");
-
-    // run with CONDA_PREFIX and CONDA_DEFAULT_ENV (unequal) and VIRTUAL_ENV set,
-    // should find child active venv
-    assert_cmd_snapshot!(case.command()
-        .current_dir(case.root().join("project"))
-        .env("CONDA_PREFIX", case.root().join("conda-env"))
-        .env("CONDA_DEFAULT_ENV", "base")
-        .env("VIRTUAL_ENV", case.root().join("myvenv")), @r"
-    success: false
-    exit_code: 1
-    ----- stdout -----
-    error[unresolved-import]: Module `package1` has no member `ActiveVenv`
-     --> test.py:2:22
-      |
-    2 | from package1 import ActiveVenv
-      |                      ^^^^^^^^^^
-    3 | from package1 import ChildConda
-    4 | from package1 import WorkingVenv
-      |
-    info: rule `unresolved-import` is enabled by default
-
-    Found 1 diagnostic
-
-    ----- stderr -----
-    WARN ty is pre-release software and not ready for production use. Expect to encounter bugs, missing features, and fatal errors.
-    ");
-
-    // run with CONDA_PREFIX and CONDA_DEFAULT_ENV (equal!) set, should find working venv
-    assert_cmd_snapshot!(case.command()
-        .current_dir(case.root().join("project"))
-        .env("CONDA_PREFIX", case.root().join("conda/envs/base"))
+        .env("CONDA_PREFIX", case.root().join("conda"))
         .env("CONDA_DEFAULT_ENV", "base"), @r"
     success: false
     exit_code: 1
@@ -1233,6 +1182,106 @@ home = ./
     WARN ty is pre-release software and not ready for production use. Expect to encounter bugs, missing features, and fatal errors.
     ");
 
+    // run with CONDA_PREFIX and CONDA_DEFAULT_ENV (unequal) and VIRTUAL_ENV set,
+    // should find child active venv
+    assert_cmd_snapshot!(case.command()
+        .current_dir(case.root().join("project"))
+        .env("CONDA_PREFIX", case.root().join("conda"))
+        .env("CONDA_DEFAULT_ENV", "base")
+        .env("VIRTUAL_ENV", case.root().join("myvenv")), @r"
+    success: false
+    exit_code: 1
+    ----- stdout -----
+    error[unresolved-import]: Module `package1` has no member `ActiveVenv`
+     --> test.py:2:22
+      |
+    2 | from package1 import ActiveVenv
+      |                      ^^^^^^^^^^
+    3 | from package1 import ChildConda
+    4 | from package1 import WorkingVenv
+      |
+    info: rule `unresolved-import` is enabled by default
+
+    Found 1 diagnostic
+
+    ----- stderr -----
+    WARN ty is pre-release software and not ready for production use. Expect to encounter bugs, missing features, and fatal errors.
+    ");
+
+    // run with CONDA_PREFIX and CONDA_DEFAULT_ENV (equal!) set, should find ChildConda
+    assert_cmd_snapshot!(case.command()
+        .current_dir(case.root().join("project"))
+        .env("CONDA_PREFIX", case.root().join("conda/envs/conda-env"))
+        .env("CONDA_DEFAULT_ENV", "conda-env"), @r"
+    success: false
+    exit_code: 1
+    ----- stdout -----
+    error[unresolved-import]: Module `package1` has no member `ChildConda`
+     --> test.py:3:22
+      |
+    2 | from package1 import ActiveVenv
+    3 | from package1 import ChildConda
+      |                      ^^^^^^^^^^
+    4 | from package1 import WorkingVenv
+    5 | from package1 import BaseConda
+      |
+    info: rule `unresolved-import` is enabled by default
+
+    Found 1 diagnostic
+
+    ----- stderr -----
+    WARN ty is pre-release software and not ready for production use. Expect to encounter bugs, missing features, and fatal errors.
+    ");
+
+    // run with _CONDA_ROOT and CONDA_PREFIX (unequal!) set, should find ChildConda
+    assert_cmd_snapshot!(case.command()
+        .current_dir(case.root().join("project"))
+        .env("CONDA_PREFIX", case.root().join("conda/envs/conda-env"))
+        .env("_CONDA_ROOT", "conda"), @r"
+    success: false
+    exit_code: 1
+    ----- stdout -----
+    error[unresolved-import]: Module `package1` has no member `ChildConda`
+     --> test.py:3:22
+      |
+    2 | from package1 import ActiveVenv
+    3 | from package1 import ChildConda
+      |                      ^^^^^^^^^^
+    4 | from package1 import WorkingVenv
+    5 | from package1 import BaseConda
+      |
+    info: rule `unresolved-import` is enabled by default
+
+    Found 1 diagnostic
+
+    ----- stderr -----
+    WARN ty is pre-release software and not ready for production use. Expect to encounter bugs, missing features, and fatal errors.
+    ");
+
+    // run with _CONDA_ROOT and CONDA_PREFIX (equal!) set, should find BaseConda
+    assert_cmd_snapshot!(case.command()
+        .current_dir(case.root().join("project"))
+        .env("CONDA_PREFIX", case.root().join("conda"))
+        .env("_CONDA_ROOT", "conda"), @r"
+    success: false
+    exit_code: 1
+    ----- stdout -----
+    error[unresolved-import]: Module `package1` has no member `BaseConda`
+     --> test.py:5:22
+      |
+    3 | from package1 import ChildConda
+    4 | from package1 import WorkingVenv
+    5 | from package1 import BaseConda
+      |                      ^^^^^^^^^
+      |
+    info: rule `unresolved-import` is enabled by default
+
+    Found 1 diagnostic
+
+    ----- stderr -----
+    WARN ty is pre-release software and not ready for production use. Expect to encounter bugs, missing features, and fatal errors.
+    ");
+
     Ok(())
 }
 
@@ -1242,15 +1291,15 @@ home = ./
 #[test]
 fn check_venv_resolution_without_working_venv() -> anyhow::Result<()> {
     let child_conda_package1_path = if cfg!(windows) {
-        "conda-env/Lib/site-packages/package1/__init__.py"
+        "conda/envs/conda-env/Lib/site-packages/package1/__init__.py"
     } else {
-        "conda-env/lib/python3.13/site-packages/package1/__init__.py"
+        "conda/envs/conda-env/lib/python3.13/site-packages/package1/__init__.py"
     };
 
     let base_conda_package1_path = if cfg!(windows) {
-        "conda/envs/base/Lib/site-packages/package1/__init__.py"
+        "conda/Lib/site-packages/package1/__init__.py"
     } else {
-        "conda/envs/base/lib/python3.13/site-packages/package1/__init__.py"
+        "conda/lib/python3.13/site-packages/package1/__init__.py"
     };
 
     let active_venv_package1_path = if cfg!(windows) {
@@ -1398,7 +1447,7 @@ home = ./
     // run with CONDA_PREFIX set, should find the child conda
     assert_cmd_snapshot!(case.command()
         .current_dir(case.root().join("project"))
-        .env("CONDA_PREFIX", case.root().join("conda-env")), @r"
+        .env("CONDA_PREFIX", case.root().join("conda/envs/conda-env")), @r"
     success: false
     exit_code: 1
     ----- stdout -----
@@ -1419,22 +1468,21 @@ home = ./
     WARN ty is pre-release software and not ready for production use. Expect to encounter bugs, missing features, and fatal errors.
     ");
 
-    // run with CONDA_PREFIX and CONDA_DEFAULT_ENV set (unequal), should find child conda
+    // run with CONDA_PREFIX and CONDA_DEFAULT_ENV set (unequal), should find base conda
     assert_cmd_snapshot!(case.command()
         .current_dir(case.root().join("project"))
-        .env("CONDA_PREFIX", case.root().join("conda-env"))
+        .env("CONDA_PREFIX", case.root().join("conda"))
         .env("CONDA_DEFAULT_ENV", "base"), @r"
     success: false
     exit_code: 1
     ----- stdout -----
-    error[unresolved-import]: Module `package1` has no member `ChildConda`
-     --> test.py:3:22
+    error[unresolved-import]: Module `package1` has no member `BaseConda`
+     --> test.py:5:22
       |
-    2 | from package1 import ActiveVenv
     3 | from package1 import ChildConda
-      |                      ^^^^^^^^^^
     4 | from package1 import WorkingVenv
     5 | from package1 import BaseConda
+      |                      ^^^^^^^^^
       |
     info: rule `unresolved-import` is enabled by default
 
@@ -1448,7 +1496,7 @@ home = ./
     // should find child active venv
     assert_cmd_snapshot!(case.command()
         .current_dir(case.root().join("project"))
-        .env("CONDA_PREFIX", case.root().join("conda-env"))
+        .env("CONDA_PREFIX", case.root().join("conda"))
         .env("CONDA_DEFAULT_ENV", "base")
         .env("VIRTUAL_ENV", case.root().join("myvenv")), @r"
     success: false
@@ -1470,10 +1518,10 @@ home = ./
     WARN ty is pre-release software and not ready for production use. Expect to encounter bugs, missing features, and fatal errors.
     ");
 
-    // run with CONDA_PREFIX and CONDA_DEFAULT_ENV (equal!) set, should find base conda
+    // run with CONDA_PREFIX and CONDA_DEFAULT_ENV (unequal!) set, should find base conda
     assert_cmd_snapshot!(case.command()
         .current_dir(case.root().join("project"))
-        .env("CONDA_PREFIX", case.root().join("conda/envs/base"))
+        .env("CONDA_PREFIX", case.root().join("conda"))
         .env("CONDA_DEFAULT_ENV", "base"), @r"
     success: false
     exit_code: 1
@@ -1494,6 +1542,55 @@ home = ./
     WARN ty is pre-release software and not ready for production use. Expect to encounter bugs, missing features, and fatal errors.
     ");
 
+    // run with _CONDA_ROOT and CONDA_PREFIX (unequal!) set, should find ChildConda
+    assert_cmd_snapshot!(case.command()
+        .current_dir(case.root().join("project"))
+        .env("CONDA_PREFIX", case.root().join("conda/envs/conda-env"))
+        .env("_CONDA_ROOT", "conda"), @r"
+    success: false
+    exit_code: 1
+    ----- stdout -----
+    error[unresolved-import]: Module `package1` has no member `ChildConda`
+     --> test.py:3:22
+      |
+    2 | from package1 import ActiveVenv
+    3 | from package1 import ChildConda
+      |                      ^^^^^^^^^^
+    4 | from package1 import WorkingVenv
+    5 | from package1 import BaseConda
+      |
+    info: rule `unresolved-import` is enabled by default
+
+    Found 1 diagnostic
+
+    ----- stderr -----
+    WARN ty is pre-release software and not ready for production use. Expect to encounter bugs, missing features, and fatal errors.
+    ");
+
+    // run with _CONDA_ROOT and CONDA_PREFIX (equal!) set, should find BaseConda
+    assert_cmd_snapshot!(case.command()
+        .current_dir(case.root().join("project"))
+        .env("CONDA_PREFIX", case.root().join("conda"))
+        .env("_CONDA_ROOT", "conda"), @r"
+    success: false
+    exit_code: 1
+    ----- stdout -----
+    error[unresolved-import]: Module `package1` has no member `BaseConda`
+     --> test.py:5:22
+      |
+    3 | from package1 import ChildConda
+    4 | from package1 import WorkingVenv
+    5 | from package1 import BaseConda
+      |                      ^^^^^^^^^
+      |
+    info: rule `unresolved-import` is enabled by default
+
+    Found 1 diagnostic
+
+    ----- stderr -----
+    WARN ty is pre-release software and not ready for production use. Expect to encounter bugs, missing features, and fatal errors.
+    ");
+
     Ok(())
 }
 
