@@ -44,7 +44,7 @@ impl TupleLength {
         TupleLength::Variable(0, 0)
     }
 
-    pub(crate) fn is_variable(self) -> bool {
+    pub(crate) const fn is_variable(self) -> bool {
         matches!(self, TupleLength::Variable(_, _))
     }
 
@@ -439,7 +439,7 @@ impl<'db> FixedLengthTuple<Type<'db>> {
                     let element_constraints =
                         self_ty.has_relation_to_impl(db, *other_ty, relation, visitor);
                     if result
-                        .intersect(db, &element_constraints)
+                        .intersect(db, element_constraints)
                         .is_never_satisfied()
                     {
                         return result;
@@ -452,7 +452,7 @@ impl<'db> FixedLengthTuple<Type<'db>> {
                     let element_constraints =
                         self_ty.has_relation_to_impl(db, *other_ty, relation, visitor);
                     if result
-                        .intersect(db, &element_constraints)
+                        .intersect(db, element_constraints)
                         .is_never_satisfied()
                     {
                         return result;
@@ -757,8 +757,7 @@ impl<'db> VariableLengthTuple<Type<'db>> {
                 // (or any other dynamic type), then the `...` is the _gradual choice_ of all
                 // possible lengths. This means that `tuple[Any, ...]` can match any tuple of any
                 // length.
-                if relation == TypeRelation::Subtyping || !matches!(self.variable, Type::Dynamic(_))
-                {
+                if !relation.is_assignability() || !self.variable.is_dynamic() {
                     return ConstraintSet::from(false);
                 }
 
@@ -774,7 +773,7 @@ impl<'db> VariableLengthTuple<Type<'db>> {
                     let element_constraints =
                         self_ty.has_relation_to_impl(db, other_ty, relation, visitor);
                     if result
-                        .intersect(db, &element_constraints)
+                        .intersect(db, element_constraints)
                         .is_never_satisfied()
                     {
                         return result;
@@ -788,7 +787,7 @@ impl<'db> VariableLengthTuple<Type<'db>> {
                     let element_constraints =
                         self_ty.has_relation_to_impl(db, other_ty, relation, visitor);
                     if result
-                        .intersect(db, &element_constraints)
+                        .intersect(db, element_constraints)
                         .is_never_satisfied()
                     {
                         return result;
@@ -832,7 +831,7 @@ impl<'db> VariableLengthTuple<Type<'db>> {
                             return ConstraintSet::from(false);
                         }
                     };
-                    if result.intersect(db, &pair_constraints).is_never_satisfied() {
+                    if result.intersect(db, pair_constraints).is_never_satisfied() {
                         return result;
                     }
                 }
@@ -858,7 +857,7 @@ impl<'db> VariableLengthTuple<Type<'db>> {
                             return ConstraintSet::from(false);
                         }
                     };
-                    if result.intersect(db, &pair_constraints).is_never_satisfied() {
+                    if result.intersect(db, pair_constraints).is_never_satisfied() {
                         return result;
                     }
                 }
