@@ -1,5 +1,5 @@
-use lsp_types::DidCloseNotebookDocumentParams;
 use lsp_types::notification::DidCloseNotebookDocument;
+use lsp_types::{DidCloseNotebookDocumentParams, NotebookDocumentIdentifier};
 
 use crate::server::Result;
 use crate::server::api::LSPResult;
@@ -21,22 +21,27 @@ impl SyncNotificationHandler for DidCloseNotebookHandler {
         _client: &Client,
         params: DidCloseNotebookDocumentParams,
     ) -> Result<()> {
-        let Ok(key) = session.key_from_url(params.notebook_document.uri.clone()) else {
-            tracing::debug!(
-                "Failed to create document key from URI: {}",
-                params.notebook_document.uri
-            );
-            return Ok(());
+        let DidCloseNotebookDocumentParams {
+            notebook_document: NotebookDocumentIdentifier { uri },
+            ..
+        } = params;
+
+        let key = match session.key_from_url(uri) {
+            Ok(key) => key,
+            Err(uri) => {
+                tracing::debug!("Failed to create document key from URI: {}", uri);
+                return Ok(());
+            }
         };
+
         session
             .close_document(&key)
             .with_failure_code(lsp_server::ErrorCode::InternalError)?;
 
         if let AnySystemPath::SystemVirtual(virtual_path) = key.path() {
-            let db = session.default_project_db_mut();
-            db.apply_changes(
+            session.apply_changes(
+                key.path(),
                 vec![ChangeEvent::DeletedVirtual(virtual_path.clone())],
-                None,
             );
         }
 
